@@ -18,11 +18,16 @@
 #endif
 #include <errno.h>
 #include <stdlib.h>
-#include <string.h>
 #ifdef HAVE_TIME_H
 #include <time.h>
 #endif
 
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+static int selinux_enabled = -1;
+#endif
+
+
 #include "b.h"
 #include "blocks.h"
 #include "main.h"
@@ -199,10 +204,15 @@ static B *bmkchn(H *chn, B *prop, long amnt, long nlines)
 	b->internal = 1;
 	b->scratch = 0;
 	b->changed = 0;
+	b->locked = 0;
+	b->ignored_lock = 0;
+	b->didfirst = 0;
 	b->count = 1;
 	b->name = NULL;
 	b->er = -3;
 	b->bof = palloc();
+	b->mod_time = 0;
+	b->check_time = time(NULL);
 	izque(P, link, b->bof);
 	b->bof->end = 0;
 	b->bof->b = b;
@@ -247,6 +257,8 @@ void brm(B *b)
 	if (b && !--b->count) {
 		if (b->changed)
 			abrerr(b->name);
+		if (b->locked && !b->ignored_lock && plain_file(b))
+			unlock_it(b->name);
 		if (b == errbuf)
 			errbuf = NULL;
 		if (b->undo)
@@ -261,6 +273,12 @@ void brm(B *b)
 	}
 }
 
+void brmall()
+{
+	while (!qempty(B, link, &bufs))
+		brm(bufs.link.next);
+}
+
 P *poffline(P *p)
 {
 	if (p->ptr) {
@@ -1950,7 +1968,7 @@ P *binsc(P *p, int c)
 /* insert zero-terminated string 's' at 'p' */
 P *binss(P *p, unsigned char *s)
 {
-	return binsm(p, s, strlen((char *)s));
+	return binsm(p, s, zlen(s));
 }
 
 /* Read 'size' bytes from file or stream.  Stops and returns amnt. read
@@ -2042,7 +2060,19 @@ unsigned char *parsens(unsigned char *s, long int *skip, long int *amnt)
 				sscanf((char *)(n + x + 1), "%ld", skip);
 		}
 	}
+	/* Don't do this here: do it in prompt buffer instead, so we're just like
+	   the shell doing it on the command line. */
+	/* n = canonical(n); */
+	return n;
+}
+
+/* Canonicalize file name: do ~ expansion */
+
+unsigned char *canonical(unsigned char *n)
+{
 #ifndef __MSDOS__
+	int x;
+	unsigned char *s;
 	if (n[0] == '~') {
 		for (x = 1; n[x] && n[x] != '/'; ++x) ;
 		if (n[x] == '/') {
@@ -2114,7 +2144,7 @@ B *bload(unsigned char *s)
 		fi = popen((char *)(n + 1), "r");
 	} else
 #endif
-	if (!strcmp(n, "-"))
+	if (!zcmp(n, US "-"))
 		fi = stdin;
 	else {
 		fi = fopen((char *)n, "r+");
@@ -2176,7 +2206,7 @@ err:
 		pclose(fi);
 	else
 #endif
-	if (strcmp(n, "-"))
+	if (zcmp(n, US "-"))
 		fclose(fi);
 
 opnerr:
@@ -2186,13 +2216,13 @@ opnerr:
 	}
 
 	/* Set name */
-	b->name = joesep((unsigned char *)strdup(s));
+	b->name = joesep(zdup(s));
 
 	/* Set flags */
 	if (error || s[0] == '!' || skip || amnt != MAXLONG) {
 		b->backup = 1;
 		b->changed = 0;
-	} else if (!strcmp(n, "-")) {
+	} else if (!zcmp(n, US "-")) {
 		b->backup = 1;
 		b->changed = 1;
 	} else {
@@ -2266,7 +2296,7 @@ B *bfind(unsigned char *s)
 		return b;
 	}
 	for (b = bufs.link.next; b != &bufs; b = b->link.next)
-		if (b->name && !strcmp(s, b->name)) {
+		if (b->name && !zcmp(s, b->name)) {
 			if (!b->orphan)
 				++b->count;
 			else
@@ -2295,7 +2325,7 @@ B *bfind_scratch(unsigned char *s)
 		return b;
 	}
 	for (b = bufs.link.next; b != &bufs; b = b->link.next)
-		if (b->name && !strcmp(s, b->name)) {
+		if (b->name && !zcmp(s, b->name)) {
 			if (!b->orphan)
 				++b->count;
 			else
@@ -2310,7 +2340,7 @@ B *bfind_scratch(unsigned char *s)
 	b->internal = 0;
 	b->rdonly = b->o.readonly;
 	b->er = error;
-	b->name = (unsigned char *)strdup((char *)s);
+	b->name = zdup(s);
 	b->scratch = 1;
 	return b;
 }
@@ -2331,7 +2361,7 @@ B *bcheck_loaded(unsigned char *s)
 		return NULL;
 	}
 	for (b = bufs.link.next; b != &bufs; b = b->link.next)
-		if (b->name && !strcmp(s, b->name)) {
+		if (b->name && !zcmp(s, b->name)) {
 			return b;
 		}
 
@@ -2413,6 +2443,8 @@ err:
  * same name as buffer or is about to get this name).
  */
 
+int break_links; /* Set to break hard links on writes */
+
 int bsave(P *p, unsigned char *s, long int size, int flag)
 {
 	FILE *f;
@@ -2434,13 +2466,47 @@ int bsave(P *p, unsigned char *s, long int size, int flag)
 #endif
 	if (s[0] == '>' && s[1] == '>')
 		f = fopen((char *)(s + 2), "a");
-	else if (!strcmp(s, "-")) {
+	else if (!zcmp(s, US "-")) {
 		nescape(maint->t);
 		ttclsn();
 		f = stdout;
 	} else if (skip || amnt != MAXLONG)
 		f = fopen((char *)s, "r+");
 	else {
+		/* Normal file save */
+		if (break_links) {
+			struct stat sbuf;
+
+			/* Try to copy permissions */
+			if (!stat((char *)s,&sbuf)) {
+				int g;
+				int en=0;
+#ifdef WITH_SELINUX
+				security_context_t se;
+				if (selinux_enabled == -1)
+					selinux_enabled = (is_selinux_enabled() > 0);
+				
+				if (selinux_enabled) {
+					if (getfilecon((char *)s, &se) < 0) {
+						error = -4;
+						goto opnerr;
+					}
+				}
+#endif
+				unlink((char *)s);
+				g = creat((char *)s, sbuf.st_mode & ~(S_ISUID | S_ISGID));
+#ifdef WITH_SELINUX
+				if (selinux_enabled) {
+					setfilecon((char *)s, &se);
+					freecon(se);
+				}
+#endif
+				close(g);
+			} else {
+				unlink((char *)s);
+			}
+		}
+
 		f = fopen((char *)s, "w");
 		norm = 1;
 	}
@@ -2475,20 +2541,20 @@ err:
 		pclose(f);
 	else
 #endif
-	if (strcmp(s, "-"))
+	if (zcmp(s, US "-"))
 		fclose(f);
 	else
 		fflush(f);
 
 	/* Update orignal date of file */
 	/* If it's not named, it's about to be */
-	if (!error && norm && flag && (!p->b->name || !strcmp((char *)s,p->b->name))) {
+	if (!error && norm && flag && (!p->b->name || !zcmp(s,p->b->name))) {
 		if (!stat((char *)s,&sbuf))
 			p->b->mod_time = sbuf.st_mtime;
 	}
 
 opnerr:
-	if (s[0] == '!' || !strcmp(s, "-")) {
+	if (s[0] == '!' || !zcmp(s,US "-")) {
 		ttopnn();
 		nreturn(maint->t);
 	}
@@ -2615,3 +2681,82 @@ RETSIGTYPE ttsig(int sig)
 		ttclsn();
 	_exit(1);
 }
+
+
+/* Create lock for a file
+   Return 0 for success or -1 for failure
+*/
+
+int lock_it(unsigned char *path,unsigned char *bf)
+{
+	unsigned char *lock_name=dirprt(path);
+	unsigned char *name=namprt(path);
+	unsigned char buf[1024];
+	unsigned char *user = (unsigned char *)getenv("USER");
+	unsigned char *host = (unsigned char *)getenv("HOSTNAME");
+	int len;
+	if (!user) user=US "me";
+	if (!host) host=US "here";
+	lock_name=vsncpy(sv(lock_name),sc(".#"));
+	lock_name=vsncpy(sv(lock_name),sv(name));
+	joe_snprintf_3((char *)buf,sizeof(buf),"%s@%s.%d",user,host,getpid());
+	if (!symlink((char *)buf,(char *)lock_name)) {
+		vsrm(lock_name);
+		vsrm(name);
+		return 0;
+	}
+	if (bf) {
+		len = readlink((char *)lock_name,(char *)bf,255);
+		if (len<0) len = 0;
+		bf[len] = 0;
+	}
+	vsrm(lock_name);
+	vsrm(name);
+	return -1;
+}
+
+void unlock_it(unsigned char *path)
+{
+	unsigned char *lock_name=dirprt(path);
+	unsigned char *name=namprt(path);
+	lock_name=vsncpy(sv(lock_name),sc(".#"));
+	lock_name=vsncpy(sv(lock_name),sv(name));
+	unlink((char *)lock_name);
+	vsrm(lock_name);
+	vsrm(name);
+}
+
+/* True if file is regular */
+
+int plain_file(B *b)
+{
+	if (b->name && zcmp(b->name,US "-") && b->name[0]!='!' && b->name[0]!='>' &&
+	    !b->scratch)
+		return 1;
+	else
+		return 0;
+}
+
+/* True if file changed under us */
+
+int check_mod(B *b)
+{
+	struct stat sbuf;
+	if (!plain_file(b))
+		return 0;
+	if (!stat((char *)b->name,&sbuf)) {
+		if (sbuf.st_mtime>b->mod_time) {
+			return 1;
+		}
+	}
+	return 0;
+}
+
+/* True if file exists */
+
+int file_exists(unsigned char *path)
+{
+	struct stat sbuf;
+	if (!path) return 0;
+	return !stat((char *)path, &sbuf);
+}
