@@ -11,10 +11,10 @@
 #include "config.h"
 #include "types.h"
 
-unsigned long hash PARAMS((char *s));
+unsigned long hash PARAMS((unsigned char *s));
 HASH *htmk PARAMS((int len));
 void htrm PARAMS((HASH *ht));
-void *htadd PARAMS((HASH *ht, char *name, void *val));
-void *htfind PARAMS((HASH *ht, char *name));
+void *htadd PARAMS((HASH *ht, unsigned char *name, void *val));
+void *htfind PARAMS((HASH *ht, unsigned char *name));
 
 #endif
