@@ -10,10 +10,10 @@
 #include "types.h"
 
 #include <stdio.h>
-#include <string.h>
 
 #include "blocks.h"
 #include "scrn.h"
+#include "charmap.h"
 #include "utils.h"
 #include "vs.h"
 #include "utf8.h"
@@ -21,18 +21,18 @@
 
 #define NOT_ENOUGH_MEMORY -11
 
-struct help *help_actual = NULL;			/* actual help screen */
+int bg_help;	/* Background color for help */
+
+struct help *help_actual = NULL;	/* actual help screen */
+struct help *help_ptr = NULL;		/* build pointer */
 
 /* 
  * Process help file
- * Returns 0 if the help file was succefully processed
- *        -1 if the help file couldn't be opened 
- *        NOT_ENOUGH_MEMORY if there is not enough memory
+ * Returns new line number
  */
 
-int help_init(unsigned char *filename)
+int help_init(FILE *fd,unsigned char *bf,int line)
 {
-	FILE *fd;					/* help file */
 	unsigned char buf[1024];			/* input buffer */
 
 	struct help *tmp;
@@ -40,87 +40,47 @@ int help_init(unsigned char *filename)
 	unsigned int hlpsiz, hlpbsz;			/* number of used/allocated bytes for tmp->text */
 	unsigned char *tempbuf;
 
-	if (!(fd = fopen((char *)filename, "r")))		/* open the help file */
-		return -1;				/* return if we couldn't open the file */
+	if (bf[0] == '{') {			/* start of help screen */
+		tmp = (struct help *) joe_malloc(sizeof(struct help));
 
-	fprintf(stderr, "Processing '%s'...", filename);
-	fflush(stderr);
+		tmp->text = NULL;
+		tmp->lines = 0;
+		hlpsiz = 0;
+		hlpbsz = 0;
+		tmp->name = vsncpy(NULL, 0, sz(bf + 1) - 1);
 
-	while (fgets((char *)buf, sizeof(buf), fd)) {
-		if (buf[0] == '{') {			/* start of help screen */
-			if (!(tmp = (struct help *) joe_malloc(sizeof(struct help)))) {
-				return NOT_ENOUGH_MEMORY;
-			}
-
-			tmp->text = NULL;
-			tmp->lines = 0;
-			hlpsiz = 0;
-			hlpbsz = 0;
-			tmp->name = vsncpy(NULL, 0, sz(buf + 1) - 1);
-
-			while ((fgets((char *)buf, sizeof(buf), fd)) && (buf[0] != '}')) {
-				bfl = strlen((char *)buf);
-				if (hlpsiz + bfl > hlpbsz) {
-					if (tmp->text) {
-						tempbuf = (unsigned char *) joe_realloc(tmp->text, hlpbsz + bfl + 1024);
-						if (!tempbuf) {
-							joe_free(tmp->text);
-							joe_free(tmp);
-							return NOT_ENOUGH_MEMORY;
-						} else {
-							tmp->text = tempbuf;
-						}
-					} else {
-						tmp->text = (unsigned char *) joe_malloc(bfl + 1024);
-						if (!tmp->text) {
-							joe_free(tmp);
-							return NOT_ENOUGH_MEMORY;
-						} else {
-							tmp->text[0] = 0;
-						}
-					}
-					hlpbsz += bfl + 1024;
-				}
-				strcpy((char *)(tmp->text + hlpsiz), (char *)buf);
-				hlpsiz += bfl;
-				++tmp->lines;
-			}
-			if (buf[0] == '}') {		/* set new help screen as actual one */
-				tmp->prev = help_actual;
-				tmp->next = NULL;
-				if (help_actual) {
-					help_actual->next = tmp;
-				}
-				help_actual = tmp;
-			} else {
-				fprintf(stderr, "\nHelp file '%s' is not properly ended with } on new line.\n", filename);
-				fprintf(stderr, "Do you want to accept incomplete help screen (y/n)?");
-				fflush(stderr);
-				fgets((char *)buf, 8, stdin);
-				if (!((buf[0] == 'y') || (buf[0] == 'Y'))) {
-					joe_free(tmp->text);
-					joe_free(tmp);
-					return 0;
+		while ((fgets((char *)buf, sizeof(buf), fd)) && (buf[0] != '}')) {
+			++line;
+			bfl = zlen(buf);
+			if (hlpsiz + bfl > hlpbsz) {
+				if (tmp->text) {
+					tempbuf = (unsigned char *) joe_realloc(tmp->text, hlpbsz + bfl + 1024);
+					tmp->text = tempbuf;
 				} else {
-					tmp->prev = help_actual;
-					tmp->next = NULL;
-					if (help_actual) {
-						help_actual->next = tmp;
-					}
-					help_actual = tmp;
+					tmp->text = (unsigned char *) joe_malloc(bfl + 1024);
+					tmp->text[0] = 0;
 				}
+				hlpbsz += bfl + 1024;
 			}
+			zcpy(tmp->text + hlpsiz, buf);
+			hlpsiz += bfl;
+			++tmp->lines;
+		}
+		tmp->prev = help_ptr;
+		tmp->next = NULL;
+		if (help_ptr) {
+			help_ptr->next = tmp;
+		} else {
+			help_actual = tmp;
+		}
+		help_ptr = tmp;
+		if (buf[0] == '}') {		/* set new help screen as actual one */
+			++line;
+		} else {
+			fprintf(stderr, "\n%d: EOF before end of help text\n",line);
 		}
 	}
-	fclose(fd);					/* close help file */
-
-	fprintf(stderr, "done\n");
-	
-	while (help_actual && help_actual->prev) {	/* move to first help screen */
-		help_actual = help_actual->prev;
-	}
-
-	return 0;
+	return line;
 }
 
 /*
@@ -134,7 +94,7 @@ struct help *find_context_help(unsigned char *name)
 	while (tmp->prev != NULL)	/* find the first help entry */
 		tmp = tmp->prev;
 
-	while (tmp != NULL && strcmp(tmp->name, name) != 0)
+	while (tmp != NULL && zcmp(tmp->name, name) != 0)
 		tmp = tmp->next;
 
 	return tmp;
@@ -147,7 +107,7 @@ void help_display(SCREEN *t)
 {
 	unsigned char *str;
 	int y, x, c, z;
-	int atr = 0;
+	int atr = BG_COLOR(bg_help);
 
 	if (help_actual) {
 		str = help_actual->text;
@@ -157,15 +117,20 @@ void help_display(SCREEN *t)
 
 	for (y = skiptop; y != t->wind; ++y) {
 		if (t->t->updtab[y]) {
-			unsigned char *start = str;
+			unsigned char *start = str, *eol;
 			int width=0;
 			int nspans=0;
 			int spanwidth;
 			int spancount=0;
 			int spanextra;
+			int len;
+
+			eol = zchr(str, '\n');
+
 			/* First pass: count no. springs \| and determine minimum width */
-			while(*str && *str!='\n')
-				if (*str++ == '\\')
+			while(*str && *str!='\n') {
+				if (*str == '\\') {
+					++str;
 					switch(*str) {
 						case 'i':
 						case 'I':
@@ -189,8 +154,13 @@ void help_display(SCREEN *t)
 							++str;
 							++width;
 					}
-				else
-					++width;
+				} else {
+					len = eol - str;
+					c = utf8_decode_fwrd(&str, &len);
+					width += joe_wcwidth(!!locale_map->type,
+							     c);
+				}
+			}
 			str = start;
 			/* Now calculate span width */
 			if (width >= t->w - 1 || nspans==0) {
@@ -203,7 +173,7 @@ void help_display(SCREEN *t)
 			/* Second pass: display text */
 			for (x = 0; x != t->w - 1; ++x) {
 				if (*str == '\n' || !*str) {
-					if (eraeol(t->t, x, y)) {
+					if (eraeol(t->t, x, y, BG_COLOR(bg_help))) {
 						return;
 					} else {
 						break;
@@ -254,17 +224,19 @@ void help_display(SCREEN *t)
 						case 0:	
 							--x;
 							continue;
-						default:
-							c = *str++;
 						}
-					} else {
-						c = *str++;
 					}
-					outatr(locale_map, t->t, t->t->scrn + x + y * t->w, 
-					             t->t->attr + x + y * t->w, x, y, c, atr);
+					len = eol - str;
+					c = utf8_decode_fwrd(&str, &len);
+
+					outatr(locale_map,
+					       t->t, t->t->scrn + x + y * t->w, 
+				       	       t->t->attr + x + y * t->w, x, y,
+					       c, atr);
+					x += (joe_wcwidth(!!locale_map->type, c) - 1);
 				}
 			}
-			atr = 0;
+			atr = BG_COLOR(bg_help);
 			t->t->updtab[y] = 0;
 		}
 
