@@ -8,13 +8,15 @@
 #include "config.h"
 #include "types.h"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
 #include <stdio.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 #include <fcntl.h>
-#include <string.h>
 #include <unistd.h>
 #ifdef HAVE_PATHS_H
 #  include <paths.h>	/* for _PATH_TMP */
@@ -26,6 +28,7 @@
 #include <stdlib.h>
 #endif
 
+#include "utils.h"
 #include "path.h"
 #include "vs.h"
 #include "va.h"
@@ -109,10 +112,10 @@ unsigned char *namepart(unsigned char *tmp, unsigned char *path)
 	unsigned char *z;
 
 	skip_drive_letter(path);
-	z = path + strlen((char *)path);
+	z = path + zlen(path);
 	while ((z != path) && (z[-1] != '/'))
 		--z;
-	return (unsigned char *)strcpy((char *)tmp, (char *)z);
+	return zcpy(tmp, z);
 }
 /********************************************************************/
 unsigned char *dirprt(unsigned char *path)
@@ -208,7 +211,7 @@ unsigned char *mktmp(unsigned char *where)
 	if (!where)
 		where = US _PATH_TMP;
 
-	namesize = strlen((char *)where) + 16;
+	namesize = zlen(where) + 16;
 	name = vsmk(namesize);	/* [G.Ghibo'] we need to use vsmk() and not malloc() as
 				   area returned by mktmp() is destroyed later with
 				   vsrm(); */
@@ -334,7 +337,7 @@ struct direct *readdir()
 		dirstate = 1;
 	}
 
-	strcpy(direc.d_name, ffblk.ff_name);
+	zcpy(direc.d_name, ffblk.ff_name);
 	for (x = 0; direc.d_name[x]; ++x)
 		direc.d_name[x] = tolower(direc.d_name[x]);
 	return &direc;
@@ -358,6 +361,21 @@ unsigned char **rexpnd(unsigned char *word)
 	return lst;
 }
 /********************************************************************/
+unsigned char **rexpnd_users(unsigned char *word)
+{
+	unsigned char **lst = NULL;
+	struct passwd *pw;
+
+	while(pw=getpwent())
+		if (rmatch(word+1, (unsigned char *)pw->pw_name)) {
+			unsigned char *t = vsncpy(NULL,0,sc("~"));
+			lst = vaadd(lst, vsncpy(sv(t),sz((unsigned char *)pw->pw_name)));
+			}
+	endpwent();
+
+	return lst;
+}
+/********************************************************************/
 int chpwd(unsigned char *path)
 {
 #ifdef __MSDOS__
@@ -373,8 +391,8 @@ int chpwd(unsigned char *path)
 	}
 	if (!path[0])
 		return 0;
-	strcpy(buf, path);
-	x = strlen(buf);
+	zcpy(buf, path);
+	x = zlen(buf);
 	while (x > 1) {
 		--x;
 		if ((buf[x] == '/') || (buf[x] == '\\'))
@@ -405,3 +423,35 @@ unsigned char *pwd(void)
 
 	return ret;
 }
+
+/* Simplify prefix by using ~ */
+/* Expects s to have trailing / */
+
+unsigned char *simplify_prefix(unsigned char *s)
+{
+	unsigned char *t = (unsigned char *)getenv("HOME");
+	unsigned char *n;
+
+#ifdef junk
+	unsigned char *org = pwd();
+	/* Normalize home */
+	if (t && !chpwd(t)) {
+		t = pwd();
+	} else {
+		t = 0;
+	}
+	chpwd(org);
+#endif
+
+	/* If current directory is prefixed with home directory, use ~... */
+	if (t && !strncmp((char *)s,(char *)t,zlen(t)) && (!s[zlen(t)] || s[zlen(t)]=='/')) {
+		n = vsncpy(NULL,0,sc("~/"));
+		/* If anything more than just the home directory, add it */
+		if (s[zlen(t)]) {
+			n = vsncpy(sv(n),s+zlen(t)+1,zlen(s+zlen(t)+1));
+		}
+	} else {
+		n = vsncpy(NULL,0,sz(s));
+	}
+	return n;
+}
