@@ -14,8 +14,12 @@
 /* BW *wmkpw(BW *bw,char *prompt,int (*func)(),char *huh,int (*abrt)(),
              int (*tab)(),void *object,int *notify);
  * Create a prompt window for the given window
+ * file_prompt flags:
+ *   bit 0: ~ expansion
+ *   bit 1: update directory
+ *   bit 2: seed with directory
  */
-BW *wmkpw PARAMS((W *w, unsigned char *prompt, B **history, int (*func) (), unsigned char *huh, int (*abrt) (), int (*tab) (), void *object, int *notify, struct charmap *map));
+BW *wmkpw PARAMS((W *w, unsigned char *prompt, B **history, int (*func) (), unsigned char *huh, int (*abrt) (), int (*tab) (), void *object, int *notify, struct charmap *map, int file_prompt));
 
 int ucmplt PARAMS((BW *bw, int k));
 
@@ -31,4 +35,9 @@ int cmplt_rtn PARAMS((MENU *m,int x,unsigned char *line));
 
 int simple_cmplt PARAMS((BW *bw,unsigned char **list));
 
+void setup_history(B **history);
+void append_history(B *hist,unsigned char *s,int len);
+void promote_history(B *hist, long line);
+void set_current_dir(unsigned char *s,int simp);
+
 #endif
