@@ -11,6 +11,7 @@
 #include "config.h"
 #include "types.h"
 
-int pmatch PARAMS((char **pieces, unsigned char *regex, int len, P *p, int n, int icase));
+int escape PARAMS((int utf8,unsigned char **a, int *b));
+int pmatch PARAMS((unsigned char **pieces, unsigned char *regex, int len, P *p, int n, int icase));
 
 #endif
