@@ -35,7 +35,6 @@
 #endif
 
 #include <errno.h>
-#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -43,6 +42,10 @@ int idleout = 1;
 
 #include "config.h"
 
+#ifdef __amigaos
+#undef SIGTSTP
+#endif
+
 /* We use the defines in sys/ioctl to determine what type
  * tty interface the system uses and what type of system
  * we actually have.
@@ -93,6 +96,8 @@ int idleout = 1;
 #include "path.h"
 #include "tty.h"
 #include "utils.h"
+#include "mouse.h"
+#include "cmd.h"
 
 /** Aliased defines **/
 
@@ -266,16 +271,50 @@ static RETSIGTYPE dotick(int unused)
 	ticked = 1;
 }
 
+extern int auto_scroll;
+
 void tickoff(void)
 {
+#ifdef HAVE_SETITIMER
+	struct itimerval val;
+	val.it_value.tv_sec = 0;
+	val.it_value.tv_usec = 0;
+	val.it_interval.tv_sec = 0;
+	val.it_interval.tv_usec = 0;
+	setitimer(ITIMER_REAL,&val,NULL);
+#else
 	alarm(0);
+#endif
 }
 
+extern int auto_scroll;
+extern int auto_trig_time;
+
 void tickon(void)
 {
+#ifdef HAVE_SETITIMER
+	struct itimerval val;
+	val.it_interval.tv_sec = 0;
+	val.it_interval.tv_usec = 0;
+	if (auto_scroll) {
+		int tim = auto_trig_time - mnow();
+		if (tim < 0)
+			tim = 1;
+		tim *= 1000;
+		val.it_value.tv_sec = 0;
+		val.it_value.tv_usec = tim;
+	} else {
+		val.it_value.tv_sec = 1;
+		val.it_value.tv_usec = 0;
+	}
+	ticked = 0;
+	joe_set_signal(SIGALRM, dotick);
+	setitimer(ITIMER_REAL,&val,NULL);
+#else
 	ticked = 0;
 	joe_set_signal(SIGALRM, dotick);
 	alarm(1);
+#endif
 }
 
 /* Open terminal */
@@ -567,17 +606,26 @@ int ttgetc(void)
 {
 	int stat;
 	long new_time;
+	int flg;
 
 
 	tickon();
 
       loop:
+      	flg = 0;
+      	/* Status line clock */
 	new_time = time(NULL);
 	if (new_time != last_time) {
 		last_time = new_time;
 		dostaupd = 1;
 		ticked = 1;
 	}
+	/* Autoscroller */
+	if (auto_scroll && mnow() >= auto_trig_time) {
+		do_auto_scroll();
+		ticked = 1;
+		flg = 1;
+	}
 	ttflsh();
 	while (winched) {
 		winched = 0;
@@ -585,7 +633,7 @@ int ttgetc(void)
 		ttflsh();
 	}
 	if (ticked) {
-		edupd(0);
+		edupd(flg);
 		ttflsh();
 		tickon();
 	}
@@ -680,9 +728,10 @@ void ttgtsz(int *x, int *y)
 #endif
 }
 
-void ttshell(unsigned char *cmd)
+int ttshell(unsigned char *cmd)
 {
 	int x, omode = ttymode;
+	int stat= -1;
 	unsigned char *s = (unsigned char *)getenv("SHELL");
 
 	if (!s) {
@@ -690,11 +739,16 @@ void ttshell(unsigned char *cmd)
 		/* return; */
 	}
 	ttclsn();
+#ifdef HAVE_FORK
 	if ((x = fork()) != 0) {
+#else
+	if ((x = vfork()) != 0) { /* For AMIGA only  */
+#endif
 		if (x != -1)
-			wait(NULL);
+			wait(&stat);
 		if (omode)
 			ttopnn();
+		return stat;
 	} else {
 		signrm();
 		if (cmd)
@@ -704,6 +758,7 @@ void ttshell(unsigned char *cmd)
 			execl((char *)s, (char *)s, NULL);
 		}
 		_exit(0);
+		return 0;
 	}
 }
 
@@ -858,7 +913,7 @@ static unsigned char *getpty(int *ptyfd)
 	static unsigned char name[32];
 	int ttyfd;
 
-        if (openpty(ptyfd, &ttyfd, name, NULL, NULL) == 0)
+        if (openpty(ptyfd, &ttyfd, (char *)name, NULL, NULL) == 0)
            return(name);
         else
 	   return (NULL);
@@ -900,23 +955,23 @@ static unsigned char *getpty(int *ptyfd)
 
 	if (ptys)
 		for (fd = 0; ptys[fd]; ++fd) {
-			strcpy((char *)ttyname, (char *)ptydir);
-			strcat((char *)ttyname, (char  *)(ptys[fd]));
+			zcpy(ttyname, ptydir);
+			zcat(ttyname, ptys[fd]);
 			if ((*ptyfd = open((char *)ttyname, O_RDWR)) >= 0) {
 				ptys[fd][0] = 't';
-				strcpy((char *)ttyname, (char *)ttydir);
-				strcat((char *)ttyname, (char *)(ptys[fd]));
+				zcpy(ttyname, ttydir);
+				zcat(ttyname, ptys[fd]);
 				ptys[fd][0] = 'p';
 				x = open((char *)ttyname, O_RDWR);
 				if (x >= 0) {
 					close(x);
 					close(*ptyfd);
-					strcpy((char *)ttyname, (char *)ptydir);
-					strcat((char *)ttyname, (char *)(ptys[fd]));
+					zcpy(ttyname, ptydir);
+					zcat(ttyname, ptys[fd]);
 					*ptyfd = open((char *)ttyname, O_RDWR);
 					ptys[fd][0] = 't';
-					strcpy((char *)ttyname, (char *)ttydir);
-					strcat((char *)ttyname, (char *)(ptys[fd]));
+					zcpy(ttyname, ttydir);
+					zcat(ttyname, ptys[fd]);
 					ptys[fd][0] = 'p';
 					return ttyname;
 				} else
@@ -1113,7 +1168,7 @@ MPX *mpxmk(int *ptyfd, unsigned char *cmd, unsigned char **args, void (*func) (/
 
 				/* If shell didn't execute */
 				joe_snprintf_1((char *)buf,sizeof(buf),"Couldn't execute shell '%s'\n",cmd);
-				write(0,(char *)buf,strlen((char *)buf));
+				write(0,(char *)buf,zlen(buf));
 				sleep(1);
 			}
 
