@@ -14,5 +14,10 @@ int ubknd PARAMS((BW *bw));
 int ukillpid PARAMS((BW *bw));
 int urun PARAMS((BW *bw));
 int ubuild PARAMS((BW *bw));
+int ugrep PARAMS((BW *bw));
+
+extern B *runhist;
+extern B *buildhist;
+extern B *grephist;
 
 #endif
