@@ -20,6 +20,7 @@
 #include "scrn.h"
 #include "utils.h"
 #include "utf8.h"
+#include "syntax.h"
 #include "w.h"
 
 extern int dspasis;		/* Set to display chars above 127 as-is */
@@ -45,8 +46,10 @@ int countmain(SCREEN *t)
 
 void wredraw(W *w)
 {
+	int x;
 	msetI(w->t->t->updtab + w->y, 1, w->h);
-	msetI(w->t->t->syntab + w->y, -1, w->h);
+	for(x=0; x!=w->h; ++x)
+		invalidate_state(w->t->t->syntab + w->y + x);
 }
 
 /* Find first window in a group */
@@ -204,7 +207,7 @@ void updall(void)
 
 	for (y = 0; y != scr->h; ++y) {
 		scr->t->updtab[y] = 1;
-		scr->t->syntab[y] = -1;
+		invalidate_state(scr->t->syntab + y);
 		}
 }
 
@@ -238,6 +241,16 @@ void scrdel(B *b, long l, long n, int flg)
 	}
 }
 
+W *watpos(SCREEN *t,int x,int y)
+{
+	W *w=t->topwin;
+	do
+		if(w->y>=0 && w->y<=y && w->y+w->h>y && w->x<=x && w->x+w->w>x)
+			return w;
+		while(w=w->link.next, w!=t->topwin);
+	return 0;
+}
+
 /* Fit as many windows on the screen as is possible beginning with the window
  * at topwin.  Give any extra space which couldn't be used to fit in another
  * window to the last text window on the screen.  This function guarentees
@@ -371,8 +384,10 @@ void wfit(SCREEN *t)
 					w->watom->resize(w->object, w->w, w->nh);
 			}
 			if (w->y == -1) {
+				int q;
 				msetI(t->t->updtab + w->ny, 1, w->nh);
-				msetI(t->t->syntab + w->ny, -1, w->nh);
+				for(q=0; q!=w->nh; ++q)
+					invalidate_state(t->t->syntab + w->ny + q);
 				}
 			w->y = w->ny;
 		} else
@@ -471,6 +486,20 @@ int wshrink(W *w)
 	return 0;
 }
 
+/* Grow window up */
+
+int wgrowup(W *w)
+{
+	return wshrink(w->link.prev->main);
+}
+
+/* Grow window down */
+
+int wgrowdown(W *w)
+{
+	return wgrow(w->link.prev->main);
+}
+
 /* Show all windows */
 
 void wshowall(SCREEN *t)
@@ -702,6 +731,8 @@ int wabort(W *w)
 
 /* Display a message and skip the next key */
 
+int bg_msg;
+
 static void mdisp(SCRN *t, int y, unsigned char *s)
 {
 	int ofst;
@@ -712,7 +743,7 @@ static void mdisp(SCRN *t, int y, unsigned char *s)
 		ofst = 0;
 	else
 		ofst = len - (t->co - 1);
-	genfmt(t, 0, y, ofst, s, 1);
+	genfmt(t, 0, y, ofst, s, BG_COLOR(bg_msg), 1);
 	t->updtab[y] = 1;
 }
 
